"""Utility for importing text files as Python lists.

This module makes it possible to treat text files of words as module
attributes.

Every file in the lists/ directory will be read and converted to a list of
strings, which will become a module attribute with the same name as the name
of the file, minus the .txt extension.

This makes adding and maintaining these lists of words very easy while also
making them very accessible inside python.

To import these lists, simply import them like any other module attribute.

For example:

    from animalid import animals
    print(len(animals))
    print(animals[0])

"""
import glob
import os
import sys


MODULE = sys.modules[__name__]
DIRECTORY = os.path.dirname(os.path.realpath(__file__))


def read_file_to_list(filename):
    """Given a file in this directory, return a set containing every line."""
    with open(os.path.join(DIRECTORY, filename), "r") as f:
        return [word.strip() for word in f.readlines()]


for word_file in glob.glob(os.path.join(DIRECTORY, "lists/*.txt")):
    word_list = read_file_to_list(word_file)
    word_key = os.path.basename(word_file).rstrip(".txt")
    setattr(MODULE, word_key, word_list)



"""Where the magic happens."""
import random

from animalid import alloys, animals, colors, fabrics, opinions, origins, shapes, sizes


FIRST_ADJECTIVES = opinions + shapes + sizes
SECOND_ADJECTIVES = alloys + colors + fabrics + origins


def generate_animal_id():
    """What it's all about."""
    return "_".join(
        [
            random.choice(FIRST_ADJECTIVES),
            random.choice(SECOND_ADJECTIVES),
            random.choice(animals),
        ]
    )



# -*- coding: utf-8 -*-
"""Pseudorandom ID generation with fun, unique animal ID's."""
from setuptools import setup, find_packages


LONG_DESCRIPTION = """
animalid is a simple library for generating fun, unique animal ID's.

ID's are strings composed of lowercase ascii letters and underscores, with a
number of adjectives followed by an animal name.

Use this library to generate unique ID's that are memorable and easy to read
and type.
"""

test_requirements = [
    "pydocstyle",
    "pytest",
    "pytest-flake8",
    "pytest-pep8",
    "pytest-mccabe",
    "pytest-cov",
]

release_requirements = ["twine", "zest.releaser[recommended]"]

setup(
    name="animalid",
    version="0.0.10",
    description=("Library for generating unique IDs with animal names."),
    url="https://github.com/alex-dr/animalid",
    author="Alex Conway",
    author_email="alex@datarobot.com",
    license="MIT",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
    keywords=["random", "id", "unique", "identifier"],
    long_description=LONG_DESCRIPTION,
    packages=find_packages(exclude=["tests"]),
    install_requires=[],
    package_data={"animalid": ["lists/*.txt"]},
    zip_safe=False,
    extras_require={
        "dev": release_requirements + test_requirements,
        "release": release_requirements,
        "testing": test_requirements,
    },
    tests_require=test_requirements,
    test_suite="tests",
)



"""Utilities for computing probabilities and such."""
from __future__ import division
import math

from animalid.random_id import FIRST_ADJECTIVES, SECOND_ADJECTIVES, animals


def compute_keyspace_size():
    """Compute how many unique keys can be generated."""
    return len(FIRST_ADJECTIVES) * len(SECOND_ADJECTIVES) * len(animals)


def compute_bits_of_entropy():
    """Compute how many bits of entropy generated phrases have."""
    return (
        math.log(len(FIRST_ADJECTIVES), 2)
        + math.log(len(SECOND_ADJECTIVES), 2)
        + math.log(len(animals), 2)
    )


def compute_probability(keyspace, trials):
    """Compute the probability of a collision.

    Uses an approximation of the full sum for computational speed.
    http://preshing.com/20110504/hash-collision-probabilities/

    :param keyspace: keyspace size
    :param trials: number of trials
    :rtype float: Probability, between 0 and 1

    """
    return 1 - math.e ** (-1 * trials * (trials - 1) / (2 * keyspace))


def _compute_expected_collisions(keyspace, trials):
    """Compute the expected number of collisions

    Uses formula from Wikipedia:
    https://en.wikipedia.org/wiki/Birthday_problem#Collision_counting

    :param keyspace: keyspace size
    :param trials: number of trials
    :rtype float: Number of expected collisions

    """
    return trials - keyspace + keyspace * ((keyspace - 1) / keyspace) ** trials


def compute_expected_collisions(trials):
    """Compute the expected number of collisions

    Uses formula from Wikipedia:
    https://en.wikipedia.org/wiki/Birthday_problem#Collision_counting

    :param trials: number of trials
    :rtype float: Number of expected collisions

    """
    keyspace = compute_keyspace_size()
    return _compute_expected_collisions(keyspace, trials)


def compute_collision_probabilities(trials):
    """Compute the probability and expected nubmer of collisions.

    :param trials: number trials to compute probabilities
    :rtype float: Probability, between 0 and 1

    """
    keyspace = compute_keyspace_size()
    return compute_probability(keyspace, trials)


def print_collision_probabilities(trials_list=[100, 1000, 10000, 100000, 1000000]):
    """Print collision probabilities for various numbers of trials

    :param trials_list: number trials to compute probabilities
    :rtype: None

    """
    print("Trials\tProbability\tExpected")
    for trials in trials_list:
        prob = compute_collision_probabilities(trials)
        expected = compute_expected_collisions(trials)
        print("{}\t{:f}\t{:f}".format(trials, prob, expected))
